//
//	ADDefaults.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADDefaults.h"
#import <stdlib.h>
#import <stdio.h>
#ifdef OPENSTEP
#import <Foundation/NSUserDefaults.h>
#import <Foundation/NSProcessInfo.h>
#endif
#import <Foundation/NSDictionary.h>
#import <Foundation/NSUtilities.h>
#import <Foundation/NSPathUtilities.h>
#import "ADUtilities.h"
#import "ADOutputModule.h"

NSString *ADCopyrightOwner     = @"ADCopyrightOwner";
NSString *ADOutputDirectory    = @"ADOutputDirectory";
NSString *ADProjectDirectory   = @"ADProjectDirectory";
NSString *ADParseSingleHeaders = @"ADParseSingleHeaders";
NSString *ADPreserveTree       = @"ADPreserveTree";
NSString *ADForceOverwrite     = @"ADForceOverwrite";
NSString *ADShowTimestamp      = @"ADShowTimestamp";
NSString *ADRunSilently        = @"ADRunSilently";
NSString *ADCopyDocumentation  = @"ADCopyDocumentation";
NSString *ADDebugLevel         = @"ADDebugLevel";
NSString *ADTemplateFilename   = @"ADTemplateFilename";
NSString *ADDocumentAllClasses = @"ADDocumentAllClasses";

void usage(NSString *error)
{
	if (error) fprintf(stderr, "Error: %s\n", [error cString]);
#ifndef OPENSTEP
	fprintf(stderr, "Usage: autodoc [files]\n");
#else
	fprintf(stderr, "Usage: %s [-force] [-copydocs] [-tree] [-nosingles] [-timestamp]\n"
"                   [-allclasses] [-help] [-version] [-silent] [-Debug #]\n"
"                   [-copyright \"Copyright String\"] [-destination \"dest path\"]\n"
"                   [-template \"template filename\"] [-format rtf|html]\n"
"                   [-project \"project directory\"] [ sourcefiles ...]\n"
"\n"
"   -force             Force document creation for up to date files.\n"
"   -copydocs          Copy documentation files found in project tree.\n"
"   -tree              Create a tree of documentation files.\n"
"   -nosingles         Exclude unpaired \".h\" and \".m\" files from documentation\n"
"   -timestamp         Insert a timestamp into the documentation\n"
"   -allclasses        Documents every class/category/protocol found in header\n"
"   -help              Show this help and exit.\n"
"   -version           Show AutoDoc version/copyright and exit.\n"
"   -silent            Operate silently;  do not generate log entries.\n"
"   -Debug #           Set debugging level to #; 0 is least, 5 is most.[ignored]\n"
"   -copyright \"...\"   Set copyrightowner in the copyright line to \"...\".\n"
"   -destination <dir> Create documentation files in directory \"..\".\n"
"   -template <file>   Use <file> as the output template\n"
"   -format rtf|html   Generate documention in either RTF or HTML format\n"
"   -project <dir>     Look for source files in the directory \"...\".\n",

	[[[NSProcessInfo processInfo] processName] cString]);
#endif

	if (error) exit(1);
	exit(0);
}

#ifndef OPENSTEP
NSDictionary *defaultDictionary = nil;

BOOL ADBoolForDefault(NSString *defaultName) {
	return [[defaultDictionary objectForKey:defaultName] isEqual:@"YES"];
}
int ADIntForDefault(NSString *defaultName) {
	return [[defaultDictionary objectForKey:defaultName] intValue];
}
NSString *ADStringForDefault(NSString *defaultName) {
	return [defaultDictionary objectForKey:defaultName];
}
#else
BOOL ADBoolForDefault(NSString *defaultName) {
	return [[NSUserDefaults standardUserDefaults] boolForKey:defaultName];
}
int ADIntForDefault(NSString *defaultName) {
	return [[NSUserDefaults standardUserDefaults] integerForKey:defaultName];
}
NSString *ADStringForDefault(NSString *defaultName) {
	return [[NSUserDefaults standardUserDefaults] stringForKey:defaultName];
}
#endif



#ifndef OPENSTEP

@interface NSArray (CStringArray)
- initWithCStringArray:(const char *const *)cStringArray count:(unsigned)count;
@end
@implementation NSArray (CStringArray)
- initWithCStringArray:(const char *const *)cStringArray count:(unsigned)count
{
	NSString *stringArray[count];
	unsigned i = 0;
	
	for (i=0; i<count; i++)
	{
		stringArray[i] = [NSString stringWithCString:cStringArray[i]];
	}
	
	return [self initWithObjects:stringArray count:count];
}
@end
#endif


NSArray *ADHandleArguments(int argc, char *argv[])
{
#ifdef OPENSTEP
	NSArray *arguments = [[NSProcessInfo processInfo] arguments];
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *argDomainDict;
#else
	NSArray *arguments =
		[[[NSArray alloc] initWithCStringArray:argv count:argc] autorelease];
#endif
	NSMutableDictionary *argDict = [NSMutableDictionary dictionary];
	NSEnumerator *argEnum = [arguments objectEnumerator];
	NSMutableArray *filesToProcess = nil;
	NSString *currArg;
	NSString *copyrightOwner = nil;
	NSString *projectDirectory = nil;

	/* Set up defaults */
	[ADOutputModule setCurrentModuleFromString:@"rtf"];
	[argDict setObject:@"YES" forKey:ADParseSingleHeaders];
//	[argDict setObject:@"/tmp" forKey:ADOutputDirectory];  // for testing

	[argEnum nextObject]; //skip over argv[0]
	
	while (currArg = [argEnum nextObject])
	{
		if ([@"-project" hasPrefix:currArg])
		{
			projectDirectory = ADNormalizePath([argEnum nextObject]);
			[argDict setObject:projectDirectory forKey:ADProjectDirectory];
		}
		else if ([@"-destination" hasPrefix:currArg])
		{
			[argDict setObject:ADNormalizePath([argEnum nextObject])
					forKey:ADOutputDirectory];
		}
		else if ([@"-copyright" hasPrefix:currArg])
		{
			copyrightOwner = [argEnum nextObject];
		}
		else if ([@"-timestamp" hasPrefix:currArg])
		{
			[argDict setObject:@"YES" forKey:ADShowTimestamp];
		}
		else if ([@"-nosingles" hasPrefix:currArg])
		{
			[argDict setObject:@"NO" forKey:ADParseSingleHeaders];
		}
		else if ([@"-copydocs" hasPrefix:currArg] || [currArg isEqual:@"-rtf"])
		{
			[argDict setObject:@"YES" forKey:ADCopyDocumentation];
		}
		else if ([@"-force" hasPrefix:currArg])
		{
			[argDict setObject:@"YES" forKey:ADForceOverwrite];
		}
		else if ([@"-tree" hasPrefix:currArg])
		{
			[argDict setObject:@"YES" forKey:ADPreserveTree];
		}
		else if ([@"-format" hasPrefix:currArg])
		{
			[ADOutputModule setCurrentModuleFromString:[argEnum nextObject]];
		}
		else if ([@"-silent" hasPrefix:currArg])
		{
			[argDict setObject:@"YES" forKey:ADRunSilently];
		}
		else if ([@"-allclasses" hasPrefix:currArg])
		{
			[argDict setObject:@"YES" forKey:ADDocumentAllClasses];
		}
		else if ([@"-template" hasPrefix:currArg])
		{
			[argDict setObject:ADNormalizePath([argEnum nextObject])
					forKey:ADTemplateFilename];
		}
		else if ([@"-Debug" hasPrefix:currArg])
		{
			[argDict setObject:[argEnum nextObject] forKey:ADDebugLevel];
		}
		else if ([@"-version" hasPrefix:currArg])
		{
			printf("AutoDoc version 2.0beta1.  Use governed by the MiscKit license.\n");
			exit(0);
		}
		else if ([@"-help" hasPrefix:currArg])
		{
			usage(nil);
		}
		else
		{
			NSString *baseFile =
				ADNormalizePath([currArg stringByDeletingPathExtension]);

			if (filesToProcess == nil) 	filesToProcess = [NSMutableArray array];
			if (![filesToProcess containsObject:baseFile])
				[filesToProcess addObject:baseFile];
		}
	}

	if (projectDirectory == nil && [filesToProcess count] == 0)
	{
		usage(@"No input files specified");
	}


	// If copyright wasn't on the command line, check the environment variable
	// use ProcessInfo for environment
	if (copyrightOwner == nil && getenv("AD_COPYRIGHT"))
		copyrightOwner = [NSString stringWithCString:getenv("AD_COPYRIGHT")];
	
	if (copyrightOwner) [argDict setObject:copyrightOwner forKey:ADCopyrightOwner];

#ifdef OPENSTEP
	/* Replace the NSArgumentDomain values with ours */
	argDomainDict = [[userDefaults volatileDomainForName:NSArgumentDomain] mutableCopy];
	[argDomainDict addEntriesFromDictionary:argDict];
	[userDefaults removeVolatileDomainForName:NSArgumentDomain];
	[userDefaults setVolatileDomain:argDomainDict forName:NSArgumentDomain];
#else
	defaultDictionary = [argDict retain];
#endif

	return filesToProcess;
}

