//
//	ADUtilities.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADUtilities.h"
#import "ADDefaults.h"
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSCharacterSet.h>
#import <Foundation/NSUtilities.h>
#import <Foundation/NSDate.h>
#ifdef OPENSTEP
#import <Foundation/NSFileManager.h>
#endif
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import "ADOutputModule.h"

#import <sys/stat.h> // should change to NSFileManager

NSCharacterSet *ADVariableNameSet()
{
	static NSMutableCharacterSet *variableSet = nil;
	
	if (!variableSet)
	{
		variableSet = [[NSCharacterSet alphanumericCharacterSet] mutableCopy];
		[variableSet addCharactersInString:@"_"];
	}
	
	return variableSet;
}

NSCharacterSet *ADNonVariableNameSet()
{
	static NSMutableCharacterSet *nonVariableSet = nil;
	
	if (!nonVariableSet)
	{
		nonVariableSet = [[ADVariableNameSet() invertedSet] retain];
	}
	
	return nonVariableSet;
}


NSRange ADRangeOfMatchedLeftParen(NSString *aString)
{
	NSRange searchRange = {0, [aString length] };
	NSCharacterSet *parenSet = [NSCharacterSet characterSetWithCharactersInString:@"()"];
	int squiggleCount = 0;
	NSRange foundRange;
	NSString *foundString;
	
	while (1)
	{
		foundRange = [aString rangeOfCharacterFromSet:parenSet
							options:NSBackwardsSearch
							range:searchRange];

		if (foundRange.length == 0) return foundRange;

		foundString = [aString substringWithRange:foundRange];
		
		if ([foundString isEqualToString:@")"]) squiggleCount++;
		else squiggleCount --;
		
		if (squiggleCount == 0) return foundRange;
		
		searchRange.length = foundRange.location;
	}
	
	return NSMakeRange(0,0);
}

NSRange ADRangeOfVariableBeforeIndex(NSString *string, unsigned lastIndex)
{
	NSRange searchRange = {0, lastIndex};
	NSRange nameStartRange, nameEndRange;
	NSRange variableRange;

	nameEndRange = [string rangeOfCharacterFromSet:ADVariableNameSet()
							options:NSBackwardsSearch
							range:searchRange];

	searchRange.length = nameEndRange.location;
	nameStartRange = [string rangeOfCharacterFromSet:ADNonVariableNameSet()
							options:NSBackwardsSearch
							range:searchRange];
	
	variableRange.location = NSMaxRange(nameStartRange);
	variableRange.length = NSMaxRange(nameEndRange) - variableRange.location;

	return variableRange;
}

NSString *ADSquashTypeString(NSString *aString)
{
	aString = [aString stringByTrimmingLeadWhitespace];
	aString = [aString stringByReplacingEverySeriesOfCharactersFromSet:
						[NSCharacterSet whitespaceAndNewlineCharacterSet]
						withString:@" "];
	return aString;
}

/* Could be an NSString method */
NSString *ADPathWithExtension(NSString *path, NSString *extension)
{
	return [[path stringByDeletingPathExtension] stringByAppendingPathExtension:extension];
}

NSString *ADNormalizePath(NSString *path)
{
#ifdef OPENSTEP
	NSFileManager *manager = [NSFileManager defaultManager];
	if (![path isAbsolutePath])
		path = [[manager currentDirectoryPath] stringByAppendingPathComponent:path];
#endif
	return [path stringByStandardizingPath];  
}

BOOL ADFileExists(NSString *filename)
{
#ifdef OPENSTEP
	NSFileManager *manager = [NSFileManager defaultManager];
	BOOL isFile, isDirectory;

	isFile = [manager fileExistsAtPath:filename isDirectory:&isDirectory];
	return (isFile && !isDirectory);
#else
	struct stat fileInfo;
	
	if (stat([filename cString], &fileInfo) != 0) return NO;
	if (fileInfo.st_mode & S_IFREG) return YES;
	return NO;
#endif
}

BOOL ADFileIsNewer(NSString *file1, NSString *file2)
{
#ifdef OPENSTEP
	NSFileManager *manager = [NSFileManager defaultManager];
	NSCalendarDate *date1 = [[manager fileAttributesAtPath:file1 traverseLink:YES] fileModificationDate];
	NSCalendarDate *date2 = [[manager fileAttributesAtPath:file2 traverseLink:YES] fileModificationDate];

	if (date1 == nil || date2 == nil) return YES;
	return ([date1 compare:date2] == NSOrderedDescending);
#endif
	return YES;  // always overwrite with 3.3
}


NSArray *ADBoldWordsArray()
{
	static NSArray *boldArray = nil;
	
	if (boldArray == nil)
	{
		boldArray = [[NSArray alloc] initWithObjects:@"id", @"self", @"super", @"nil", nil];
	}
	
	return boldArray;
}

/* Does the formatting for _{...} type of markup. */
static NSString *ADFormatListMarkup(NSString *string, ADOutputModule *outputModule)
{
	NSMutableString *newString = [NSMutableString stringWithCapacity:[string length]];
	unsigned stringLength = [string length];
	NSRange searchRange = NSMakeRange(0, stringLength);
	NSRange foundRange;
	NSRange betweenRange = NSMakeRange(0, 0);
	NSString *listRegex = @"_{ *([^} ]+) *([^}]*)} *";
	NSString *listGroupRegex = [NSString stringWithFormat:@"(%@[ \n\r\t]*)+", listRegex];
	
	foundRange = [string rangeOfRegex:listGroupRegex range:searchRange];
	while (foundRange.location != NSNotFound)
	{
		NSString *listsString;
		betweenRange.length = foundRange.location - betweenRange.location;
		[newString appendString:[string substringWithRange:betweenRange]];
		
		[newString appendString:[outputModule listPrefixString]];
		listsString = [string substringWithRange:foundRange];
		listsString = [listsString stringByReplacingEveryOccurrenceOfRegex:listRegex
							withString:[outputModule listItemReplaceString]
							options:MiscUseMatchSubstitutions];
		[newString appendString:listsString];
		[newString appendString:[outputModule listPostfixString]];
		
		betweenRange.location = NSMaxRange(foundRange);
		searchRange = NSMakeRange(betweenRange.location, stringLength - betweenRange.location);
		foundRange = [string rangeOfRegex:listGroupRegex range:searchRange];
	}
	
	[newString appendString:[string substringFromIndex:betweenRange.location]];

	return newString;
}

/*"
 * Formats the string, converting autodoc-style markup into the appropriate
 * output format.
"*/
/*
 * Maybe it would be a good idea to have another lexer program to parse the
 * comment string... it would probably be a bit more flexible, and we could
 * maybe have different lexers to handle different kinds of markup in
 * comments in case we want to allow something other than the #{}, %{}, _{}
 * stuff we have now.
 */
NSString *ADFormatString(NSString *string, NSArray *italicWordsArray)
{
	ADOutputModule *outputModule = [ADOutputModule currentModule];
	NSArray *boldWordsArray = ADBoldWordsArray();
	NSString *startBold = [outputModule startBold];
	NSString *stopBold  = [outputModule stopBold];
	NSString *startItal = [outputModule startItalics];
	NSString *stopItal  = [outputModule stopItalics];
	NSString *boldReplace1   = [NSString stringWithFormat:@"%@$1%@",   startBold, stopBold];
	NSString *boldReplace2   = [NSString stringWithFormat:@"$1%@$2%@", startBold, stopBold];
	NSString *italicReplace1 = [NSString stringWithFormat:@"%@$1%@",   startItal, stopItal];
	NSString *italicReplace2 = [NSString stringWithFormat:@"$1%@$2%@", startItal, stopItal];
	NSEnumerator *wordEnum;
	NSString     *currWord;

	string = [outputModule preProcessString:string];

    /* boldify text of the form #{the text} */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"#{([^}]+)}"
						withString:boldReplace1
						options:MiscUseMatchSubstitutions];
	
    /* italicize text of the form %{the text} */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"%{([^}]+)}"
						withString:italicReplace1
						options:MiscUseMatchSubstitutions];

    /* boldify words which start with a "#" in front of them */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"([^\\\\])#((\\w|:)+)"
						withString:boldReplace2
						options:MiscUseMatchSubstitutions];

    /* italicize words which start with a "%" in front of them */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"([^\\\\])%((\\w|:)+)"
						withString:italicReplace2
						options:MiscUseMatchSubstitutions];


    /* boldify words in boldWordsArray */
	wordEnum = [boldWordsArray objectEnumerator];
	while (currWord = [wordEnum nextObject])
	{
		NSString *regex = [NSString stringWithFormat:@"(\\b%@\\b)", currWord];
		string = [string stringByReplacingEveryOccurrenceOfRegex:regex
							withString:boldReplace1
							options:MiscUseMatchSubstitutions];
	}
	
    /* italicize words in italicWordsArray */
	wordEnum = [italicWordsArray objectEnumerator];
	while (currWord = [wordEnum nextObject])
	{
		NSString *regex = [NSString stringWithFormat:@"(\\b%@\\b)", currWord];
		if ([currWord length] == 0) continue;   //Make sure
		string = [string stringByReplacingEveryOccurrenceOfRegex:regex
							withString:italicReplace1
							options:MiscUseMatchSubstitutions];
	}
	
    /* Reformat '\%' and '\#' to just '%' and '#' */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"\\\\([#%])"
							withString:@"$1" options:MiscUseMatchSubstitutions];

	/* Make double newline markers into paragraph separators */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"\n\n[\n\t ]*"
							withString:[outputModule paragraphSeparator]];

	/* Format _{...} type of markup */
	string = ADFormatListMarkup(string, outputModule);
	
	string = [outputModule postProcessString:string];
	
	return string;
}

NSString *ADFormatValueString(NSString *string)
{
	ADOutputModule *outputModule = [ADOutputModule currentModule];

	string = [string stringByReplacingEveryOccurrenceOfRegex:@"\n[\t ]*" withString:@"\n"];
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"[\t ]+" withString:@" "];
	
	if ([string containsString:@"{"])
	{
		NSString *indent = @"\t";
		NSArray *lineArray = [string componentsSeparatedByString:@"\n"];
		NSEnumerator *lineEnum = [lineArray objectEnumerator];
		NSString *line;
		NSMutableArray *newArray = [NSMutableArray arrayWithCapacity:[lineArray count]];

		while (line = [lineEnum nextObject])
		{
			if ([line containsString:@"{"])
			{
				NSString *replace;
				indent = [indent stringByAppendingString:@"\t"];
				replace = [NSString stringWithFormat:@"{%@$1", indent];
				line = [line stringByReplacingRegex:@"{ *([^ ]*)"
							withString:replace
							options:MiscUseMatchSubstitutions];
			}
			else if ([line containsString:@"}"])
			{
				indent = [indent stringByReplacingRegex:@"\t" withString:@""];
				line = [NSString stringWithFormat:@"%@%@", indent, line];
			}
			else
			{
				line = [NSString stringWithFormat:@"%@%@", indent, line];
			}
			
			[newArray addObject:line];
		}
		
		string = [newArray componentsJoinedByString:@"\n"];
	}
	
	string = [outputModule preProcessString:string];
	string = [string stringByReplacingEveryOccurrenceOfString:@"\n"
					withString:[outputModule lineSeparator]];
	string = [outputModule postProcessString:string];
	
	return string;
}

