//
//	AutoDocMain.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADDefaults.h"
#import <Foundation/NSDate.h>
#import <Foundation/NSUtilities.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSAutoreleasePool.h>
#ifdef OPENSTEP
#import <Foundation/NSFileManager.h>
#endif
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import <MiscFoundation.subproj/regexpr.h>
#import "ADFile.h"
#import "ADInterface.h"
#import "ADMethod.h"
#import "ADUtilities.h"
#import "ADOutputModule.h"
#import <stdio.h>

void ADHandleInputFile(NSString *filename);
NSString *ADOutputFilenameForInputFile(NSString *headerFilename);
NSString *ADOutputFilenameForFile(ADFile *aFile);
BOOL ADShouldOutputFile(NSString *header, NSString *source, NSString *targetFilename);

int main(int argc, char *argv[])
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	NSEnumerator *fileEnum;
	NSString	 *filename;
	NSArray      *filesToProcess = ADHandleArguments(argc, argv);

	re_set_syntax(RE_SYNTAX_AWK);

	fileEnum = [filesToProcess objectEnumerator];
	while (filename = [fileEnum nextObject])
	{
		filename = ADPathWithExtension(filename, @"h");
		if (!ADFileExists(filename))
			NSLog(@"Error: '%@' not found", filename);
		else
			ADHandleInputFile(filename);
	}

#ifdef OPENSTEP
	{
	NSDirectoryEnumerator *directoryEnum = nil;
	NSString *projectDirectory = ADStringForDefault(ADProjectDirectory);
	NSString *outputDirectory  = ADStringForDefault(ADOutputDirectory);
	BOOL	copyDocFiles = ADBoolForDefault(ADCopyDocumentation);

	if (projectDirectory)
		directoryEnum = [[NSFileManager defaultManager] enumeratorAtPath:projectDirectory];

	/* Recurse through entire project */
	while (filename = [directoryEnum nextObject])
	{
		NSString *fullFilename = [projectDirectory stringByAppendingPathComponent:filename];
		NSString *currExtension = [filename pathExtension];
		NSString *outputExtension = [[ADOutputModule currentModule] fileExtension];
		
		if ([currExtension isEqualToString:@"h"])
		{
			ADHandleInputFile([projectDirectory stringByAppendingPathComponent:filename]);
		}
		else if (copyDocFiles && outputDirectory &&
				[currExtension isEqualToString:outputExtension])
		{
			NSString *outFile = ADOutputFilenameForInputFile(fullFilename);
			[[NSFileManager defaultManager] copyPath:filename toPath:outFile handler:nil];
		}
	}
	}
#endif

	[pool release];
	return 0;
}

void ADHandleInputFile(NSString *filename)
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	BOOL	 parseSingles = ADBoolForDefault(ADParseSingleHeaders);
	BOOL	 outputAll    = ADBoolForDefault(ADDocumentAllClasses);
	NSString *headerFilename = ADPathWithExtension(filename, @"h");
	NSString *sourceFilename = ADPathWithExtension(filename, @"m");
	NSString *outputFilename = ADOutputFilenameForInputFile(filename);
	ADOutputModule *outputModule = [ADOutputModule currentModule];
	ADFile   *fileContents;

	if (!ADFileExists(headerFilename)) return;

	if (!ADFileExists(sourceFilename)) sourceFilename = ADPathWithExtension(filename, @"mdoc");
	if (!ADFileExists(sourceFilename)) sourceFilename = nil;

	if (!sourceFilename && !parseSingles) return;

	if (!outputAll && !ADShouldOutputFile(headerFilename, sourceFilename, outputFilename))
		return;
	
	fileContents = [[ADFile alloc] initWithHeaderFilename:headerFilename
	                                       sourceFilename:sourceFilename];

	if (!outputAll)
	{
		[outputModule outputContents:fileContents toFile:outputFilename];
	}
	else
	{
		NSEnumerator *fileEnum = [[fileContents subfileArray] objectEnumerator];
		ADFile       *currFile;

		while (currFile = [fileEnum nextObject])
		{
			outputFilename = ADOutputFilenameForFile(currFile);
			if (ADShouldOutputFile(headerFilename, sourceFilename, outputFilename))
				[outputModule outputContents:currFile toFile:outputFilename];
		}
	}

	[fileContents release];
	[pool release];
}

BOOL ADShouldOutputFile(NSString *headerFilename, NSString *sourceFilename, NSString *targetFilename)
{
    	if (ADBoolForDefault(ADForceOverwrite)) return YES;

	if (ADFileIsNewer(headerFilename, targetFilename) ||
	    (sourceFilename && ADFileIsNewer(sourceFilename, targetFilename)))
	{
		return YES;
	}

	return NO;
}

NSString *ADOutputFilenameForFile(ADFile *aFile)
{
	NSString *inputFile = [aFile headerFilename];
	
	if (ADBoolForDefault(ADDocumentAllClasses) && [aFile hasMainInterface])
	{
		NSString *path = [[aFile headerFilename] stringByDeletingLastPathComponent];
		NSString *base = [[aFile mainInterface] uniqueFileBasename];
		
		inputFile = [path stringByAppendingPathComponent:base];
	}

	return ADOutputFilenameForInputFile(inputFile);
}

NSString *ADOutputFilenameForInputFile(NSString *headerFilename)
{
#if OPENSTEP
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSString	*projectDirectory = ADStringForDefault(ADProjectDirectory);
	BOOL		preserveTree      = ADBoolForDefault(ADPreserveTree);
	BOOL		beSilent          = ADBoolForDefault(ADRunSilently);
#else
/* -project not implemented on 3.3 */
	NSString *projectDirectory = nil;	
	BOOL preserveTree = NO;
#endif
	NSString	*outputDirectory  = ADStringForDefault(ADOutputDirectory);
	NSString *outputFile = nil;
	ADOutputModule *outputModule = [ADOutputModule currentModule];
	
	if (projectDirectory && outputDirectory && preserveTree &&
		[headerFilename hasPrefix:projectDirectory])
	{
		NSString *relativePath = [headerFilename substringAfterString:projectDirectory];
		outputFile = [outputDirectory stringByAppendingString:relativePath];
	}
	else if (outputDirectory)
	{
		outputFile = [outputDirectory stringByAppendingPathComponent:
										[headerFilename lastPathComponent]];
	}
	else
	{
		outputFile = headerFilename;
	}

	outputFile = ADPathWithExtension(outputFile, [outputModule fileExtension]);

#if OPENSTEP
	outputDirectory = [outputFile stringByDeletingLastPathComponent];
	if (![fileManager fileExistsAtPath:outputDirectory])
	{
		if (!beSilent) NSLog(@"Creating directory %@", outputDirectory);
		if (![fileManager createDirectoryAtPath:outputDirectory attributes:nil])
			NSLog(@"Failed to create directory %@", outputDirectory);
	}
#endif

	return outputFile;
}

