//
//	ADInterface.m
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADInterface.h"
#import "ADMethod.h"
#import "ADComment.h"
#import "ADUtilities.h"
#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>
#import <Foundation/NSUtilities.h>

@implementation ADInterface

/*"
 * ADInterface is the abstract superclass for Objective-C entities, i.e.
 * classes, categories, and protocols.  It holds information common to
 * those entities.
 * 
 * The conformedProtocolArray holds the list of protocols the interface
 * conforms to as an NSArray of NSStrings.
 * 
 * The declarationArray holds the list of ADMethods and ADComments declared
 * in the header.  A comment signals the start of a new grouping of methods
 * in the "Method Types" section of the documentation.
 * 
 * The methodDictionary holds all of the methods declared in the header,
 * keyed by the -methodName of the ADMethod.  This is primarily to make the
 * matching of methods to their descriptions in the source file easier.
"*/

/*"
 * The designated initilizer of ADInterface.  Just intializes a couple of
 * instance variables.
"*/
- init
{
	[super init];

	declarationArray = [[NSMutableArray alloc] init];
	methodDictionary = [[NSMutableDictionary alloc] init];

	return self;
}

- (void)dealloc
{
	[conformedProtocolArray release];
	[declarationArray release];
	[methodDictionary release];
	[super dealloc];
}

/*"
 * Returns the main title to be used in the generated documentation.  This
 * is the class name for classes and protocols, and a string including both
 * the class name and category name for categories. Subclasses must
 * override.
"*/
- (NSString *)title
{
	return @"Subclass responsibility!";
}

/*"
 * Returns a string describing what kind of an interface this is, i.e.
 * "Class", "Category" , or "Protocol".  This is used in the generated
 * documentation.  Subclasses must override.
"*/
- (NSString *)interfaceTitle
{
	return @"Subclass responsibility!";
}

/*"
 * Returns the file basename to be used with this interface when generating
 * documentation for every class/category/protocol (since the base of the
 * input filename can't be used in this situation).  Subclasses must
 * override.
"*/
- (NSString *)uniqueFileBasename
{
	return @"SubclassResponsibility";
}


/*"
 * Returns the protocols conformed to as an NSArray of NSStrings.  If the
 * receiving interface doesn't conform to any protocols, returns nil.
"*/
- (NSArray *)conformedProtocols
{
	return conformedProtocolArray;
}

/*"
 * Sets the array of conformed protocols to protocolNameArray.  Mainly for
 * use by the parser.
"*/
- (void)setConformedProtocols:(NSArray *)protocolNameArray
{
	SETIVAR(conformedProtocolArray, protocolNameArray);
}

- (NSString *)conformedProtocolString
{
	if ([conformedProtocolArray count] >0)
	{
		return [NSString stringWithFormat:@" <%@>",
				[[self conformedProtocols] componentsJoinedByString:@", "]];
	}
	
	return @"";
}

- (NSArray *)declarationArray;
{
	return declarationArray;
}

/*"
 * Adds declarations from array.  Each "declaration" should be an ADMethod
 * or an ADComment.  Mainly for use by the parser.
"*/
- (void)addDeclarationsFromArray:(NSArray *)array
{
	NSEnumerator *declarationEnum = [array objectEnumerator];
	id			declaration;
	
	while (declaration = [declarationEnum nextObject])
	{
		[self addDeclaration:declaration];
	}
}

/*"
 * Add a single declaration, which should be either an ADMethod or an
 * ADComment.  Adds anObject to declarationArray, and if it's an ADMethod,
 * adds it to methodDictionary as well.  Mainly for use by the parser.
"*/
- (void)addDeclaration:(id)anObject
{
	[declarationArray addObject:anObject];
	
	if ([anObject isKindOfClass:[ADMethod class]])
	{
		[methodDictionary setObject:anObject forKey:[anObject methodName]];
	}
}

- (ADMethod *)methodNamed:(NSString *)methodName
{
	return [methodDictionary objectForKey:methodName];
}

/*"
 * Returns an array of all the ADMethods in declarationArray that represent
 * class methods.  The array will be sorted by method name.
"*/
- (NSArray *)classMethods
{
	NSEnumerator *methodEnum = [[self declarationArray] objectEnumerator];
	NSMutableArray *classMethodArray = [NSMutableArray array];
	ADMethod *currMethod;
	
	while (currMethod = [methodEnum nextObject])
	{
		if ([currMethod isKindOfClass:[ADMethod class]] && [currMethod isClassMethod])
			[classMethodArray addObject:currMethod];
	}
	
	return [classMethodArray sortedArrayUsingSelector:@selector(compareMethod:)];
}


/*"
 * Returns an array of all the ADMethods in declarationArray that represent
 * instance methods.  The array will be sorted by method name.
"*/
- (NSArray *)instanceMethods
{
	NSEnumerator *methodEnum = [[self declarationArray] objectEnumerator];
	NSMutableArray *instanceMethodArray = [NSMutableArray array];
	ADMethod *currMethod;
	
	while (currMethod = [methodEnum nextObject])
	{
		if ([currMethod isKindOfClass:[ADMethod class]] && ![currMethod isClassMethod])
			[instanceMethodArray addObject:currMethod];
	}
	
	return [instanceMethodArray sortedArrayUsingSelector:@selector(compareMethod:)];
}

/*"
 * We store the method declarations and the comments that logically "group"
 * them together in the same array (declarationArray).  This method breaks
 * them all up into an array of NSDictionaries, where each dictionary has
 * two keys: "comment", which holds the ADComment for the method group, and
 * "methodArray", which is the subarray of method to be grouped together in
 * the documentation.
"*/
- (NSArray *)declarationGroupArray
{
	NSEnumerator *declEnum = [[self declarationArray] objectEnumerator];
	NSMutableArray *groupArray = [NSMutableArray array];
	id currDecl;
	NSMutableDictionary *currGroup;
	NSMutableArray *currMethodArray = nil;
	
	while (currDecl = [declEnum nextObject])
	{
		if ([currDecl isKindOfClass:[ADComment class]])
		{
			currGroup = [NSMutableDictionary dictionary];
			currMethodArray = [NSMutableArray array];
			[currGroup setObject:currDecl forKey:@"comment"];
			[currGroup setObject:currMethodArray forKey:@"methodArray"];
			[groupArray addObject:currGroup];
		}
		else if (currMethodArray == nil)
		{
			currGroup = [NSMutableDictionary dictionary];
			currMethodArray = [NSMutableArray array];
			[currGroup setObject:currMethodArray forKey:@"methodArray"];
			[groupArray addObject:currGroup];
			[currMethodArray addObject:currDecl];
		}
		else
		{
			[currMethodArray addObject:currDecl];
		}
	}
	
	return groupArray;
}

/*"
 * Takes the ADMethods in methodArray, finds the corresponding methods in
 * the receiver, and sets their entityDescription to that of the given
 * ADMethods.  Mainly for use by the parser, for setting the method
 * descriptions found in the source file.
"*/
- (void)setMethodDescriptionsFromArray:(NSArray *)methodArray
{
	NSEnumerator *methodEnum = [methodArray objectEnumerator];
	ADMethod	 *currMethod;
	ADMethod	 *myMethod;
	
	while (currMethod = [methodEnum nextObject])
	{
		if (myMethod = [self methodNamed:[currMethod methodName]])
		{
			[myMethod setEntityDescription:[currMethod entityDescription]];
		}
	}
}

@end
