//
//	ADOutputModule.m
//		Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//				All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import "ADOutputModule.h"
#import <Foundation/NSUtilities.h>
#import <Foundation/NSPathUtilities.h>
#import <Foundation/NSObjCRuntime.h>
#import <Foundation/NSBundle.h>
#import <Foundation/NSData.h>
#import <Foundation/NSAutoreleasePool.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscMerge.subproj/MiscMergeEngine.h>
#import <MiscMerge.subproj/MiscMergeTemplate.h>
#import "ADDefaults.h"
#import "ADUtilities.h"
#import "ADFile.h"

static ADOutputModule *currentModule;

@interface ASCIIOutputModule : ADOutputModule
@end
@interface RTFOutputModule : ADOutputModule
@end
@interface HTMLOutputModule : ADOutputModule
@end

static NSDictionary *_outputMapDictionary()
{
	return [@"{ "
				@"rtf = RTFOutputModule; "
				@"html = HTMLOutputModule; "
				@"txt = ASCIIOutputModule; "
				@"ascii = ASCIIOutputModule; "
			@"}"
			propertyList];
}

@implementation ADOutputModule

+ (ADOutputModule *)currentModule
{
	return currentModule;
}

+ (void)setCurrentModuleFromString:(NSString *)aString
{
	NSString *className = [_outputMapDictionary() objectForKey:[aString lowercaseString]];

	if (className == nil) className = @"RTFOutputModule";
	
	[currentModule release];
	currentModule = [[NSClassFromString(className) alloc] init];
}

/* Subclasses should override */
- (NSString *)fileExtension		{ return @""; }
- (NSString *)templateFilename	{ return @""; }

- (NSString *)startBold		{ return @""; }
- (NSString *)stopBold		{ return @""; }
- (NSString *)startItalics	{ return @""; }
- (NSString *)stopItalics	{ return @""; }
- (NSString *)paragraphSeparator { return @""; }
- (NSString *)lineSeparator { return @""; }

- (NSString *)listPrefixString 		{ return @""; 		}
- (NSString *)listPostfixString		{ return @""; 		}
- (NSString *)listItemReplaceString { return @"$1 $2";	}

/* Subclasses can override */
- (NSString *)postProcessString:(NSString *)string
{
	return string;
}
- (NSString *)preProcessString:(NSString *)string
{
	return string;
}

- (MiscMergeTemplate *)template
{
	/* We can't change output formats, so a single template instance is enough */
	static MiscMergeTemplate *template = nil;
	
	if (template == nil)
	{
		NSString *templateFile = ADStringForDefault(ADTemplateFilename);

		if (templateFile == nil)
		{
			/* FIXME - should go through search path */
			templateFile = [[NSBundle mainBundle]
								pathForResource:[self templateFilename] ofType:nil];
		}
		if (templateFile == nil)
		{
			NSLog(@"%@: Can't find template file %@", [self class], [self templateFilename]);
		}
		template = [[MiscMergeTemplate alloc] initWithContentsOfFile:templateFile];
	}
	
	return template;
}

- (void)outputContents:(ADFile *)fileContents toFile:(NSString *)outputFilename
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	MiscMergeEngine	  *merger = [[[MiscMergeEngine alloc] init] autorelease];
	NSData			  *outputData;

	[merger setTemplate:[self template]];
	[merger setMainObject:fileContents];
	
	outputData = [[merger execute:nil] dataUsingEncoding:[NSString defaultCStringEncoding]];

	outputFilename = ADPathWithExtension(outputFilename, [self fileExtension]);
	if (!ADBoolForDefault(ADRunSilently))
		NSLog(@"Generating %@", [outputFilename lastPathComponent]);
	[outputData writeToFile:outputFilename atomically:YES];
	[pool release];
}

@end

@implementation ASCIIOutputModule

- (NSString *)fileExtension		 	{ return @"txt"; }
- (NSString *)paragraphSeparator 	{ return @"\n\n"; }
- (NSString *)lineSeparator		 	{ return @"\n"; }
- (NSString *)listItemReplaceString { return @"\t$1\t$2\n";	}

/* Until I have an ASCII template, this'll do */
- (void)outputContents:(ADFile *)fileContents toFile:(NSString *)outputFilename
{
	NSData *outputData = [[fileContents description]
								dataUsingEncoding:[NSString defaultCStringEncoding]];

	outputFilename = ADPathWithExtension(outputFilename, [self fileExtension]);
	[outputData writeToFile:outputFilename atomically:YES];
}

@end

#define BOLDSTART @"__START_BOLD_TOKEN"
#define ITALSTART @"__START_ITAL_TOKEN"

@implementation RTFOutputModule

- (NSString *)fileExtension		 { return @"rtf"; }
- (NSString *)templateFilename	 { return @"RTFTemplate.rtf"; }

//- (NSString *)startBold		{ return @"\\b "; }
- (NSString *)startBold		{ return BOLDSTART; } //see below
- (NSString *)stopBold		{ return @"\\b0 "; }
//- (NSString *)startItalics	{ return @"\\i "; }
- (NSString *)startItalics	{ return ITALSTART; } // see below
- (NSString *)stopItalics	{ return @"\\i0 "; }
- (NSString *)paragraphSeparator { return @"\\\n\\fs16 \\\n\\fs28 "; }
- (NSString *)lineSeparator { return @"\\\n"; }

- (NSString *)listItemReplaceString
{
	/* We handle the BEGSQ and ENDSQ in the post processing */
	return @"BEGSQ\\pard \\li7648 \\fi-5131 \\ri1007 \\ql \\tx7648 "
				@"\\tx10180 \\fs16 \\\n \\fs28  $1\t$2 ENDSQ\n ";
}

- (NSString *)postProcessString:(NSString *)string
{
    /*
     * Reformat curly braces so they don't prematurely end the rtf doc.
     */
	string = [string stringByReplacingEveryOccurrenceOfRegex:@"[{}]"
						withString:@"\\$0"
						options:MiscUseMatchSubstitutions];

	/* This is necessary because otherwise the rtf { and } would be replaced by the above */
	string = [string stringByReplacingEveryOccurrenceOfString:@"BEGSQ" withString:@"{"];
	string = [string stringByReplacingEveryOccurrenceOfString:@"ENDSQ" withString:@"}"];

    /*
     * Because the bold and italics start strings have a space in them,
     * they can confuse later parsing if they are put in too early.  So, we
     * insert a token, and put in the actual RTF markup here.
     */
	string = [string stringByReplacingEveryOccurrenceOfString:BOLDSTART withString:@"\\b "];
	string = [string stringByReplacingEveryOccurrenceOfString:ITALSTART withString:@"\\i "];
	return string;
}

@end

@implementation HTMLOutputModule

- (NSString *)fileExtension		 { return @"html"; }
- (NSString *)templateFilename	 { return @"HTMLTemplate.html"; }

- (NSString *)startBold		{ return @"<B>"; }
- (NSString *)stopBold		{ return @"</B>"; }
- (NSString *)startItalics	{ return @"<I>"; }
- (NSString *)stopItalics	{ return @"</I>"; }
- (NSString *)paragraphSeparator { return @"<P>\n\n"; }
- (NSString *)lineSeparator { return @"<BR>\n"; }

- (NSString *)listPrefixString	{	return @"<TABLE BORDER=\"3\">\n";	}
- (NSString *)listPostfixString	{	return @"</TABLE>\n";	}
- (NSString *)listItemReplaceString	{	return @"\t<TR><TD>$1</TD><TD>$2</TD></TR>\n";	}

- (NSString *)preProcessString:(NSString *)string
{
    /*
     * Replace certain characters with their HTML entities.  I'm not
     * handling any other entities rignt now; only normal ascii ones.
     * 
     * Make sure the '&' one is done first.
     * 
     * This does disallow HTML markup in comments...hm.
     */
	string = [string stringByReplacingEveryOccurrenceOfString:@"&" withString:@"&amp;"];
	string = [string stringByReplacingEveryOccurrenceOfString:@"<" withString:@"&lt;"];
	string = [string stringByReplacingEveryOccurrenceOfString:@">" withString:@"&gt;"];
	string = [string stringByReplacingEveryOccurrenceOfString:@"\"" withString:@"&quot;"];
	
	return string;
}

@end
