/*
//	HeaderTokenizer.l
//	  Written by Carl Lindberg Copyright 1998 by Carl Lindberg.
//                     All rights reserved.
//		This notice may not be removed from this source code.
//
//	This header is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"License.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
*/	

%{

#import <Foundation/NSString.h>
#import <Foundation/NSArray.h>
#import <Foundation/NSData.h>
#import <MiscFoundation.subproj/NSString+MiscRegex.h>
#import <MiscFoundation.subproj/NSString+MiscAdditions.h>

#import "ADClass.h"
#import "ADComment.h"
#import "ADProtocol.h"
#import "ADIvar.h"
#import "ADCategory.h"
#import "ADMethod.h"

#define YYSTRING [NSString stringWithCString:yytext length:yyleng]
#define DATASTRING [NSString stringWithCString:[accumData bytes] length:[accumData length]]
#define ACCUMTEXT() [accumData appendBytes:yytext length:yyleng]
#define RESETDATA()	if (!accumData) accumData = [[NSMutableData alloc] init]; \
                    [accumData setLength:0]

	typedef struct _MethodArg {
		NSString *argName;
		NSString *argType;
		NSString *varName;
	} MethodArg;

#import "HeaderGrammar.m.h"

#define yylval HeaderGrammarlval

static BOOL foundClass = NO;
static int numSquiggles = 0;
static NSMutableData *accumData = nil;

%}

%option never-interactive
%option noyywrap
%option stack
%option yylineno
	// These two remove warnings
%option noyy_top_state
%option noreject
	//%option debug

ident				[a-zA-Z_][a-zA-Z0-9_]*
ccomment			("/*"([^*]|"*"+[^*/])*"*"+"/")
objccomment			("//"[^\n]*\n?)
whitespacechar		[ \t\r\n]
whitespace			{whitespacechar}*
whitespaceexist		{whitespacechar}+
statictype			(\([^:;@+-]*\))
methodstart			^{whitespace}[+-]{whitespace}{statictype}?
methodargument		":"{whitespace}{statictype}?{whitespace}{ident}


%s CLASS
%s METHOD
%s VARIABLES
%x INTYPEDEF
%x INENUM

%%

{ccomment}				{
							NSString *commentString = YYSTRING;
							
							/* Autodoc actually allows comments like '/****"  */
							if ([commentString containsRegex:@"\\`/[*]+\""] &&
								[commentString containsRegex:@"\"[*]+/\\'"])
							{
								yylval.stringVal = commentString;
								return AUTODOCCOMMENT;
							}
							if ([commentString containsString:@"$Id:"] &&
								YYSTATE == INITIAL)
							{
								yylval.stringVal = commentString;
								return COPYRIGHTCOMMENT;
							}
						}
<*>{objccomment}			;

<*>"#"{whitespace}(import|include){whitespace}"<"[^>]+">"	;
<*>"#"{whitespace}(import|include){whitespace}"\""[^\"]+"\""	;
<*>"#"{whitespace}(ifdef|ifndef|if|else|elsif|endif|undef)[^\n]*\n	;

@class{whitespace}{ident}({whitespace},{whitespace}{ident})*{whitespace}";"	;

(extern|[A-Z]+_(EXTERN|EXPORT)){whitespaceexist}[^;]+")"{whitespace}";"	{
							if (!foundClass && YYSTATE == INITIAL)
							{
								yylval.stringVal = YYSTRING; 
								return FUNCTION;
								
								/* Warning avoidance */
								unput(' ');			// always needed
								goto find_rule;		// needed if %option yylineno
							}
						}

(extern|[A-Z]+_(EXTERN|EXPORT)){whitespaceexist}[^;]+";"			{
							if (!foundClass && YYSTATE == INITIAL)
							{
								yylval.stringVal = YYSTRING; 
								return GLOBALVAR;
							}
						}

(static|inline){whitespaceexist}[^;]+";"			;

"#"{whitespace}define{whitespace}{ident}"("(\\\n|.)+  {
							if (!foundClass && YYSTATE == INITIAL)
							{
								yylval.stringVal = YYSTRING; 
								return MACRO;
							}
						}

"#"{whitespace}define{whitespace}{ident}(\\\n|.)+  {
							if (!foundClass && YYSTATE == INITIAL)
							{
								yylval.stringVal = YYSTRING; 
								return CONSTANT;
							}
						}

typedef{whitespaceexist}	{	yy_push_state(INTYPEDEF);
								RESETDATA();
								ACCUMTEXT();
							}
<INTYPEDEF>"{"			{	ACCUMTEXT(); numSquiggles++;	}
<INTYPEDEF>"}"			{	ACCUMTEXT(); numSquiggles--;	}
<INTYPEDEF>{ccomment}	;
<INTYPEDEF>";"			{	ACCUMTEXT();
							if (numSquiggles == 0)
							{
								yy_pop_state();
								yylval.stringVal = DATASTRING;
								if (!foundClass && YYSTATE == INITIAL) return TYPEDEF;
							}
						}
<INTYPEDEF>[^;{}/#]+	{	ACCUMTEXT(); }
<INTYPEDEF>[/#]+		{	ACCUMTEXT(); }

obsolete{whitespaceexist}	;	/* A "keyword" that occasionally pops up in NeXT's headers */

enum{whitespaceexist}	{
							yy_push_state(INENUM);
							RESETDATA();
							ACCUMTEXT();
						}
<INENUM>"'{'"				{	ACCUMTEXT();	}
<INENUM>"'}'"				{	ACCUMTEXT();	}
<INENUM>"{"				{	ACCUMTEXT(); numSquiggles++;	}
<INENUM>"}"				{	ACCUMTEXT(); numSquiggles--;	}
<INENUM>{ccomment}		;
<INENUM>";"				{	ACCUMTEXT();
							if (numSquiggles == 0)
							{
								yy_pop_state();
								yylval.stringVal = DATASTRING;
								if (!foundClass && YYSTATE == INITIAL) return ENUM;
							}
						}
<INENUM>[^;{}'/#]+		{	ACCUMTEXT(); }
<INENUM>['/#]+			{	ACCUMTEXT(); }

@interface				{	BEGIN(CLASS); return INTERFACE; }
@protocol				{	BEGIN(CLASS); return PROTOCOL; }

<CLASS>[:()<>,]			{	return yytext[0]; }

<CLASS>@end				{	BEGIN(INITIAL); return END;	}

<CLASS>"{"				{	BEGIN(VARIABLES); return yytext[0]; }

<VARIABLES>"}"			{	BEGIN(CLASS); return yytext[0]; }

						/* Maybe we shouldn't document @private ivars at all */
<VARIABLES>(@public|@protected|@private)	;

<VARIABLES>[^ \t\r\n;/{}@#][^;{}]+";"	{yylval.stringVal = YYSTRING; return VARDECL; }

<VARIABLES>struct{whitespace}({ident}{whitespace})?"{"		{return BEGINSTRUCT; }
<VARIABLES>union{whitespace}({ident}{whitespace})?"{"		{return BEGINUNION; }

<VARIABLES>"}"{whitespace}{ident}{whitespace}";"	{yylval.stringVal = YYSTRING; return ENDSTRUCT; }

{ident}					{	if (YYSTATE != INITIAL)
							{
								yylval.stringVal = YYSTRING; return IDENT;
							}
						}

<CLASS>[+-]				{	BEGIN(METHOD); yylval.charVal = yytext[0];	return METHODSTART; }

<METHOD>[:,]			{	return yytext[0]; }

<METHOD>{statictype}	{	yylval.stringVal = YYSTRING; return STATICTYPE;	}
<METHOD>"..."			{	yylval.stringVal = YYSTRING; return ELLIPSIS; }

<METHOD>";"				{	BEGIN(CLASS);	return yytext[0]; }

{whitespaceexist}				;
.|\n						;


%%

void ADSetHeaderFile(FILE *file)
{
	yyin = file;
}

